==> AnnBuilder_1.3.3.tar.gz: AnnBuilder/R/GO.R <==
# Constructors an object of GO
#

GO <- function(srcUrl = getSrcUrl("GO", xml = TRUE), parser = "",
               baseFile = ""){

    new("GO", srcUrl = srcUrl, parser = parser, baseFile = baseFile)
}

# The GO site provides an XML document for the molecular function,
# biological process, and cellular component of genes. The basic XML
# structure is something like:
# <go:term>
#   <go:accession>GO:000xxx</go:accession>
#   <go:name>a string for the function, process, or component</go:name>
#   <go:isa rdf:resource="http://www.geneontology.org/go#GO:000xxx/>
#   .
#   .
#   .
# <go:term>
#
# The XML document istself does not differentiats among the fuction,
# biological process, and cellular component of genes as a go:name tag
# is used for the fucntion, process, and component of genes. To
# determine whether a go:name tag is for the function, process, or
# component of a given gene idenfied by a GO accession number, the
# go:isa tag has to be used to move up a tree to find the correct
# category.
#
# For example, gene ontology has an accession number of GO:0003673,
# mocular function GO:0003674, cellular componenet GO:0005574, and
# biological process GO:0008150. The mocular function, cellular
# component, and biological process all have the tag, <go:isa
# rdf:resource="http://www.geneontology.org/go#GO:0003673/>,
# indicating that they all belong to gene ontology. Similarly, if a
# gene (say GO:0001284) has a tag  <go:isa
# rdf:resource="http://www.geneontology.org/go#GO:0008150/>, we know
# that the text inclosed by the go:name tag should be considered as a
# biological process. Often, the go:isa tag of a gene may not directly
# reference a top level GO accession (e. g. the number for mocular
# function, cellular component, or biological process) but to another
# gene (say GO:0001284), the ontology of that gene will be traced back
# to biological process through the reference to GO:0008150 by gene
# GO:001284.
#
# The GOXMLParser reads in the go.xml document from Gene Ontology site,
# creates a table named gooriginal with three columns (GOID,
# ParentGOID, and Ontology) in the database, and then populates the
# table with data parsed from the XML document. Both the GOID and
# ParentGOID are keys for the table as a given GOID may have multiple
# parent GOIDs.The handlers used currently may need to be modified
# accordingly if Gene Ontology site changes the DTD of the XML document.
#
# Copyright 2001 J. Zhang, all rights reserved.
#
GOXMLParser <- function(fileName) {
    # Flags that will be checked by function text
    isGOID <- FALSE
    isOnto <- FALSE
    # Keeps a temporary value
    GOID <- NA
    parentID <- NA
    ontology <- NA
    association <- NA
    evidence <- NA
    # Keep the desired data
    GOData <- NULL
    # Reset the values when done with a go term
    reset <- function(){
        GOID <<- NA
        parentID <<- NA
        ontology <<- NA
        association <<- NA
        evidence <<- NA
    }
    # Get the GO id or ontology for an element
    getText <- function(value){
        if(isGOID){
            GOID <<- value
            isGOID <<- FALSE
        }else if(isOnto){
            ontology <<- value
            isOnto <<- FALSE
        }
    }
    # Parent GO id of an element is given as an attribute that can be
    # get this way
    getParentGOID <- function(attrs, assoType){
        if(is.na(parentID)){
            association <<- paste(parseIsa(attrs), assoType, sep = "@")
            parentID <<- parseIsa(attrs)
        }else{
            parentID <<- paste(parentID, parseIsa(attrs), sep = ";")
            names(assoType) <- parseIsa(attrs)
            association <<- paste(association,
                              paste(parseIsa(attrs), assoType, sep = "@"),
                                    sep = ";")
        }
    }
    # parse the isa tag and get only the GO accession number
    parseIsa <- function(isaTag){
        return(gsub(".*http.*#(.*)", "\\1", isaTag))
    }
    # Merge the parsed data
    mergeData <- function(){
        if(is.null(GOData)){
            GOData <<- c(GOID, parentID, ontology, association)
        }else{
            GOData <<- rbind(GOData, c(GOID, parentID, ontology, association))
        }
    }
    # Event handlers for the parser
    getHandler <- function(){
        # Gets the value for desired element
        startElement <- function(name, attrs){
            switch(name,
                   "go:accession" = isGOID <<- TRUE,
                   "go:name" = isOnto <<- TRUE,
                   "go:part_of" = getParentGOID(attrs, "partof"),
                   "go:is_a" = getParentGOID(attrs, "isa"))
        }
        # Write the data when entries for a GO term ends
        endElement <- function(name,...){
            if(name == "go:term"){
                mergeData()
                reset()
            }
        }
        text <- function(x){
            getText(x)
        }

        list(startElement = startElement, endElement = endElement,
             text = text)
    }
    xmlEventParse(fileName, handlers = getHandler())
    colnames(GOData) <- c("goid", "parents", "ontology", "association")
    # Remove GO:isa and GO:part-of and return
    return(GOData[!is.element(GOData[,1], c("GO:isa", "GO:partof")), ])
}

getChildNodes <- function(goid, goData){
    return(goData[grep(goid, goData[, 2]), 1])
}

getOffspringNodes <- function(goid, goData, keepTree = FALSE){

    offspring <- NULL

    getChildren <- function(goid){
#        print(goData[goData[,1] == goid,])
        return(goData[grep(goid, goData[, 2]), 1])
#        return(getChildNodes(goData, goid))
    }

    repeat{
        temp <- unique(unlist(sapply(goid, getChildren), use.names = FALSE))
        if(length(temp) > 0){
            goid <- temp
            if(is.null(offspring)){
                offspring <- unique(temp)
            }else{
                offspring <- unique(c(offspring, temp))
            }
        }else{
            break
        }
    }
    return(offspring)
}

getParentNodes <- function(goid, goData, sep = ";"){
    return(unlist(strsplit(goData[(goData[,1] == goid), 2], sep),
                  use.names = FALSE))
}

getAncestors <- function(goid, goData, sep = ";", keepTree = FALSE,
                         top = "GO:0003673"){
    ancestors <- NULL
    getAncestor <- function(goid){
         return(unlist(strsplit(goData[(goData[,1] == goid), 2], sep),
                  use.names = FALSE))
#        return(getParentNodes(goData, goid, sep = sep))
    }

    repeat{
        temp <- unique(unlist(sapply(goid, getAncestor), use.names = FALSE))
        temp <- temp[temp != top]
        if(length(temp) > 0){
            goid <- temp
            if(is.null(ancestors)){
                if(keepTree){
                    ancestors <- list(1, unique(temp))
                }else{
                    ancestors <- unique(temp)
                }
            }else{
                if(keepTree){
                    ancestors[[length(ancestors) + 1]] <- unique(temp)
                }else{
                    ancestors <- unique(c(ancestors, temp))
                }
            }
        }else{
            if(keepTree){
                ancestors[[length(ancestors) + 1]] <- top
            }else{
                ancestors <- unique(c(ancestors, top))
            }
            break
        }
    }
    return(ancestors)
}

getTopGOid <- function(what = c("MF", "BP", "CC", "GO")){
    what = match.arg(what)
    switch(what,
           MF = return("GO:0003674"),
           BP = return("GO:0008150"),
           CC = return("GO:0005575"),
           GO = return("GO:0003673"),
           stop("Invalid argument imput!"))
}

mapGO2Category <- function(goData){
    goNCategory <- cbind(getTopGOid("GO"), "GO")
    goGroups <- getGOGroupIDs()
    for(i in names(goGroups)){
        goids <- c(as.vector(goGroups[i]),
                   getOffspringNodes(goGroups[i], goData))
        # Map GO ids to the three ontology categories.
        goNCategory <- rbind(goNCategory, cbind(goids, i))
    }
    return(goNCategory)
}

getGOGroupIDs <- function(onto = FALSE){
    if(!onto){
        return(c(MF = "GO:0003674", CC = "GO:0005575", BP = "GO:0008150"))
    }else{
        return("GO:0003673")
    }
}

==> AnnBuilder_1.4.2.tar.gz: AnnBuilder/R/GO.R <==
# Constructors an object of GO
#

GO <- function(srcUrl = getSrcUrl("GO", xml = TRUE), parser = "",
               baseFile = "", built){

    new("GO", srcUrl = srcUrl, parser = parser, baseFile = baseFile,
        built = ifelse(missing(built), getSrcBuilt("GO"), built))
}

# The GO site provides an XML document for the molecular function,
# biological process, and cellular component of genes. The basic XML
# structure is something like:
# <go:term>
#   <go:accession>GO:000xxx</go:accession>
#   <go:name>a string for the function, process, or component</go:name>
#   <go:isa rdf:resource="http://www.geneontology.org/go#GO:000xxx/>
#   .
#   .
#   .
# <go:term>
#
# The XML document istself does not differentiats among the fuction,
# biological process, and cellular component of genes as a go:name tag
# is used for the fucntion, process, and component of genes. To
# determine whether a go:name tag is for the function, process, or
# component of a given gene idenfied by a GO accession number, the
# go:isa tag has to be used to move up a tree to find the correct
# category.
#
# For example, gene ontology has an accession number of GO:0003673,
# mocular function GO:0003674, cellular componenet GO:0005574, and
# biological process GO:0008150. The mocular function, cellular
# component, and biological process all have the tag, <go:isa
# rdf:resource="http://www.geneontology.org/go#GO:0003673/>,
# indicating that they all belong to gene ontology. Similarly, if a
# gene (say GO:0001284) has a tag  <go:isa
# rdf:resource="http://www.geneontology.org/go#GO:0008150/>, we know
# that the text inclosed by the go:name tag should be considered as a
# biological process. Often, the go:isa tag of a gene may not directly
# reference a top level GO accession (e. g. the number for mocular
# function, cellular component, or biological process) but to another
# gene (say GO:0001284), the ontology of that gene will be traced back
# to biological process through the reference to GO:0008150 by gene
# GO:001284.
#
# The GOXMLParser reads in the go.xml document from Gene Ontology site,
# creates a table named gooriginal with three columns (GOID,
# ParentGOID, and Ontology) in the database, and then populates the
# table with data parsed from the XML document. Both the GOID and
# ParentGOID are keys for the table as a given GOID may have multiple
# parent GOIDs.The handlers used currently may need to be modified
# accordingly if Gene Ontology site changes the DTD of the XML document.
#
# Copyright 2001 J. Zhang, all rights reserved.
#
GOXMLParser <- function(fileName) {
    # Flags that will be checked by function text
    isGOID <- FALSE
    isOnto <- FALSE
    # Keeps a temporary value
    GOID <- NA
    parentID <- NA
    ontology <- NA
    association <- NA
    evidence <- NA
    # Keep the desired data
    GOData <- NULL
    # Reset the values when done with a go term
    reset <- function(){
        GOID <<- NA
        parentID <<- NA
        ontology <<- NA
        association <<- NA
        evidence <<- NA
    }
    # Get the GO id or ontology for an element
    getText <- function(value){
        if(isGOID){
            GOID <<- value
            isGOID <<- FALSE
        }else if(isOnto){
            ontology <<- value
            isOnto <<- FALSE
        }
    }
    # Parent GO id of an element is given as an attribute that can be
    # get this way
    getParentGOID <- function(attrs, assoType){
        if(is.na(parentID)){
            association <<- paste(parseIsa(attrs), assoType, sep = "@")
            parentID <<- parseIsa(attrs)
        }else{
            parentID <<- paste(parentID, parseIsa(attrs), sep = ";")
            names(assoType) <- parseIsa(attrs)
            association <<- paste(association,
                              paste(parseIsa(attrs), assoType, sep = "@"),
                                    sep = ";")
        }
    }
    # parse the isa tag and get only the GO accession number
    parseIsa <- function(isaTag){
        return(gsub(".*http.*#(.*)", "\\1", isaTag))
    }
    # Merge the parsed data
    mergeData <- function(){
        if(is.null(GOData)){
            GOData <<- c(GOID, parentID, ontology, association)
        }else{
            GOData <<- rbind(GOData, c(GOID, parentID, ontology, association))
        }
    }
    # Event handlers for the parser
    getHandler <- function(){
        # Gets the value for desired element
        startElement <- function(name, attrs){
            switch(name,
                   "go:accession" = isGOID <<- TRUE,
                   "go:name" = isOnto <<- TRUE,
                   "go:part_of" = getParentGOID(attrs, "partof"),
                   "go:is_a" = getParentGOID(attrs, "isa"))
        }
        # Write the data when entries for a GO term ends
        endElement <- function(name,...){
            if(name == "go:term"){
                mergeData()
                reset()
            }
        }
        text <- function(x){
            getText(x)
        }

        list(startElement = startElement, endElement = endElement,
             text = text)
    }
    xmlEventParse(fileName, handlers = getHandler())
    colnames(GOData) <- c("goid", "parents", "ontology", "association")
    # Remove GO:isa and GO:part-of and return
    return(GOData[!is.element(GOData[,1], c("GO:isa", "GO:partof")), ])
}

getChildNodes <- function(goid, goData){
    return(goData[grep(goid, goData[, 2]), 1])
}

getOffspringNodes <- function(goid, goData, keepTree = FALSE){

    offspring <- NULL

    getChildren <- function(goid){
#        print(goData[goData[,1] == goid,])
        return(goData[grep(goid, goData[, 2]), 1])
#        return(getChildNodes(goData, goid))
    }

    repeat{
        temp <- unique(unlist(sapply(goid, getChildren), use.names = FALSE))
        if(length(temp) > 0){
            goid <- temp
            if(is.null(offspring)){
                if(keepTree){
                    offspring <- list()
                    offspring[[1]] <- as.vector(temp)
                }else{
                    offspring <- temp
                }
            }else{
                if(keepTree){
                    offspring[[length(offspring) + 1]] <- temp
                }else{
                    offspring <- unique(c(offspring, temp))
                }
            }
        }else{
            break
        }
    }
    if(is.null(offspring)){
        return(NA)
    }else{
        return(offspring)
    }
}

getParentNodes <- function(goid, goData, sep = ";"){
    return(unlist(strsplit(goData[(goData[,1] == goid), 2], sep),
                  use.names = FALSE))
}

getAncestors <- function(goid, goData, sep = ";", keepTree = FALSE,
                         top = "GO:0003673"){
    ancestors <- NULL
    getAncestor <- function(goid){
         return(unlist(strsplit(goData[(goData[,1] == goid), 2], sep),
                  use.names = FALSE))
#        return(getParentNodes(goData, goid, sep = sep))
    }

    repeat{
        temp <- unique(unlist(sapply(goid, getAncestor), use.names = FALSE))
        temp <- temp[temp != top]
        if(length(temp) > 0){
            goid <- temp
            if(is.null(ancestors)){
                if(keepTree){
                    ancestors <- list()
                    ancestors[[1]] <- as.vector(temp)
                }else{
                    ancestors <- temp
                }
            }else{
                if(keepTree){
                    ancestors[[length(ancestors) + 1]] <- temp
                }else{
                    ancestors <- unique(c(ancestors, temp))
                }
            }
        }else{
            if(keepTree){
                ancestors[[length(ancestors) + 1]] <- top
            }else{
                ancestors <- unique(c(ancestors, top))
            }
            break
        }
    }
    if(is.null(ancestors)){
        return(NA)
    }else{
        return(ancestors)
    }
}

getTopGOid <- function(what = c("MF", "BP", "CC", "GO")){
    what = match.arg(what)
    switch(what,
           MF = return("GO:0003674"),
           BP = return("GO:0008150"),
           CC = return("GO:0005575"),
           GO = return("GO:0003673"),
           stop("Invalid argument imput!"))
}

mapGO2Category <- function(goData){
    goNCategory <- cbind(getTopGOid("GO"), "GO")
    goGroups <- getGOGroupIDs()
    for(i in names(goGroups)){
        goids <- c(as.vector(goGroups[i]),
                   getOffspringNodes(goGroups[i], goData))
        # Map GO ids to the three ontology categories.
        goNCategory <- rbind(goNCategory, cbind(goids, i))
    }
    return(goNCategory)
}

getGOGroupIDs <- function(onto = FALSE){
    if(!onto){
        return(c(MF = "GO:0003674", CC = "GO:0005575", BP = "GO:0008150"))
    }else{
        return("GO:0003673")
    }
}
